package com.bytesw.bytebot.etl.batch.writer;

import com.bytesw.bytebot.etl.batch.beans.DeleteDataSensBean;
import com.bytesw.bytebot.etl.batch.beans.DynaBean;
import com.bytesw.bytebot.etl.beans.IntentBean;
import com.bytesw.bytebot.etl.dao.IntentRepository;
import com.bytesw.bytebot.etl.enums.EventTypeEnum;
import com.bytesw.bytebot.etl.services.ProcessMessageService;
import com.bytesw.bytebot.etl.utils.JsonUtils;
import com.bytesw.xdf.exception.NotFoundException;
import com.bytesw.xdf.multitenant.core.ThreadLocalStorage;
import com.bytesw.bytebot.etl.services.DeleteSensMessageService;
import com.bytesw.bytebot.service.provider.TwilioService;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.log4j.Log4j2;
import org.springframework.batch.core.ExitStatus;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.StepExecutionListener;
import org.springframework.batch.item.ItemWriter;
import com.bytesw.bytebot.etl.batch.beans.DeleteDataSensRegistryBean;
import org.springframework.beans.factory.annotation.Autowired;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;


@Log4j2
public class DataSensibleJPAWriter implements ItemWriter<DynaBean>, StepExecutionListener {

    @Autowired
    IntentRepository intentRepository;

    @Getter @Setter
    private DeleteSensMessageService service;

    @Getter @Setter
    private List<String> intentByAgent;

    @Getter @Setter
    private DeleteDataSensBean agent;


    @Override
    @Transactional
    public void write(List<? extends DynaBean> list) throws Exception {
        for (DynaBean dynaBean : list) {
            log.debug("LLegue ACA");
            String json = (String) dynaBean.get("data");
            String event = (String) JsonUtils.getFieldFromJson(json, "$.event");
            if (EventTypeEnum.USER.getName().equals(event)) {
                String toAgent = (String) JsonUtils.getFieldFromJson(json, "$.metadata.To");
                if (agent.getChannelValue().equals(toAgent)) {
                    String intent = (String) JsonUtils.getFieldFromJson(json, "$.parse_data.intent.name");
                    if (intentByAgent.indexOf(intent) >= 0) {
                        log.debug("Llegue aca");
                        String SmSMessageSid = (String) JsonUtils.getFieldFromJson(json, "$.metadata.SmsMessageSid");

                        //Nuevos parametros para el llamado del metodo
                        //@TODO evaluar si transformar a MAP

                        Optional<IntentBean> intenId = this.intentRepository.findIntenNameById(intent, agent.getAgenId());
                        if (!intenId.isPresent()){
                            //Exception
                        }

                        String senderId = (String) JsonUtils.getFieldFromJson(json, "$.metadata.sender_id");;

                        List<DeleteDataSensRegistryBean> deleteSensibleBean = service.deleteMessage(agent.getAgenId()
                                ,SmSMessageSid, intenId.get().getId(), senderId);
                    }
                }
            }
        }
        //service.saveDataSensControl(eventBean.getId(), eventBean.getSenderId());
    }

    @Override
    public void beforeStep(StepExecution stepExecution) {
        String tenantId = stepExecution.getJobParameters().getString("tenantId");
        ThreadLocalStorage.setTenantName(tenantId);
    }

    @Override
    public ExitStatus afterStep(StepExecution stepExecution) {
        return stepExecution.getExitStatus();
    }
}
